@@ -17,6 +17,7 @@
 
 
 #include "common.h"
+#include "option.h"
 #include <assert.h>
 #if HAVE_GETTEXT
 # include <libintl.h>
@@ -29,7 +30,6 @@
 #include "builtin.h"
 #include "exec.h"
 #include "job.h"
-#include "option.h"
 #include "plist.h"
 #include "redir.h"
 #include "sig.h"
@@ -332,10 +332,8 @@ int parse_shell_options(int argc, void *const *argv,
 		goto set_positional_parameters;
 	    }
 
-	    /* parse a long option */
 	    result = parse_long_option(argv, enable, shell_invocation);
 	} else {
-	    /* parse short options */
 	    result = parse_short_option(argv, enable, shell_invocation);
 	}
 	if (result != Exit_SUCCESS)
@@ -383,7 +381,7 @@ int parse_short_option(void *const *argv, bool enable,
 		if (optname == NULL) {
 		    xerror(0, Ngt("the -%lc option requires an argument"),
 			    (wint_t) L'o');
-		    return Exit_ERROR;
+		    return special_builtin_syntax_error(Exit_ERROR);
 		}
 	    }
 
@@ -434,7 +432,7 @@ int parse_option_character(
     }
 
     xerror(0, Ngt("`%ls' is not a valid option"), (wchar_t[]) { opt, L'\0' });
-    return Exit_ERROR;
+    return special_builtin_syntax_error(Exit_ERROR);
 }
 
 /* Parses the long option at `xoptind' in the specified arguments and
@@ -544,7 +542,7 @@ int handle_search_result(plist_T *options, void *const *argv, bool enable,
 	case 0:
 	    pl_destroy(options);
 	    xerror(0, Ngt("`%ls' is not a valid option"), optstr);
-	    return Exit_ERROR;
+	    return special_builtin_syntax_error(Exit_ERROR);
 	case 1:
 	    if (noshelloptindex > 0) {
 		const struct option_T *opt = options->contents[0];
@@ -568,7 +566,8 @@ int handle_search_result(plist_T *options, void *const *argv, bool enable,
 				if (optarg == NULL) {
 				    xerror(0, Ngt("the --%ls option requires "
 						"an argument"), opt->longopt);
-				    return Exit_ERROR;
+				    return special_builtin_syntax_error(
+					    Exit_ERROR);
 				}
 				break;
 			    default:
@@ -581,7 +580,7 @@ int handle_search_result(plist_T *options, void *const *argv, bool enable,
 		    if (eq != NULL) {
 			xerror(0, Ngt("%ls: the --%ls option does not take "
 				    "an argument"), optstr, opt->longopt);
-			return Exit_ERROR;
+			return special_builtin_syntax_error(Exit_ERROR);
 		    }
 		    optarg = NULL;
 		}
@@ -602,7 +601,7 @@ int handle_search_result(plist_T *options, void *const *argv, bool enable,
 		    ((const struct xgetopt_T *) options->contents[i])->longopt);
 #endif
 	    pl_destroy(options);
-	    return Exit_ERROR;
+	    return special_builtin_syntax_error(Exit_ERROR);
     }
 }
 
@@ -615,7 +614,7 @@ int set_shell_option(const struct option_T *option, bool enable,
 	xerror(0, Ngt("the %ls option cannot be changed "
 		    "once the shell has been initialized"),
 		option->longopt);
-	return Exit_ERROR;
+	return special_builtin_syntax_error(Exit_ERROR);
     }
 
     *option->optp = enable;
@@ -768,17 +767,51 @@ wchar_t *get_hyphen_parameter(void)
     return wb_towcs(&buf);
 }
 
+#if YASH_ENABLE_HELP
+
+/* Prints a list of all shell options to the standard output.
+ * Returns true iff successful. */
+bool print_shopts_body(bool include_normal_options)
+{
+    if (include_normal_options)
+	if (!print_option_list(normal_options))
+	    return false;
+
+    for (const struct option_T *o = shell_options; o->longopt != NULL; o++) {
+	if (!xprintf("\t"))
+	    return false;
+
+	if (o->shortopt_enable != L'\0') {
+	    if (!xprintf("-%lc", o->shortopt_enable))
+		return false;
+	} else if (o->shortopt_disable != L'\0') {
+	    if (!xprintf("+%lc", o->shortopt_disable))
+		return false;
+	} else {
+	    if (!xprintf("  "))
+		return false;
+	}
+
+	if (!xprintf("       -o %ls\n", o->longopt))
+	    return false;
+    }
+
+    return true;
+}
+
+#endif /* YASH_ENABLE_HELP */
+
 
 /********** Built-in **********/
 
-static const struct xgetopt_T all_help_options[] = {
+const struct xgetopt_T all_help_options[] = {
     { L'a', L"all",  OPTARG_NONE, true,  NULL, },
+#if YASH_ENABLE_HELP
     { L'-', L"help", OPTARG_NONE, false, NULL, },
+#endif
     { L'\0', NULL, 0, false, NULL, },
 };
-
-const struct xgetopt_T *const all_option  = &all_help_options[0];
-const struct xgetopt_T *const help_option = &all_help_options[1];
+/* Note: `help_option' is defined as (&all_help_options[1]). */
 
 
 static int set_builtin_print_current_settings(void);
@@ -825,5 +858,15 @@ int set_builtin_print_restoring_commands(void)
     return Exit_SUCCESS;
 }
 
+#if YASH_ENABLE_HELP
+const char set_help[] = Ngt(
+"set shell options and positional parameters"
+);
+const char set_syntax[] = Ngt(
+"\tset [option...] [--] [new_positional_parameter...]\n"
+"\tset -o|+o  # print current settings\n"
+);
+#endif
+
 
 /* vim: set ts=8 sts=4 sw=4 noet tw=80: */
